@@ -1,5 +1,9 @@
 package wifi
 
+import (
+	"time"
+)
+
 // A Client is a type which can access WiFi device actions and statistics
 // using operating system-specific operations.
 type Client struct {
@@ -55,3 +59,21 @@ func (c *Client) BSS(ifi *Interface) (*BSS, error) {
 func (c *Client) StationInfo(ifi *Interface) ([]*StationInfo, error) {
 	return c.c.StationInfo(ifi)
 }
+
+// SurveyInfo retrieves the survey information about a WiFi interface.
+func (c *Client) SurveyInfo(ifi *Interface) ([]*SurveyInfo, error) { return c.c.SurveyInfo(ifi) }
+
+// SetDeadline sets the read and write deadlines associated with the connection.
+func (c *Client) SetDeadline(t time.Time) error {
+	return c.c.SetDeadline(t)
+}
+
+// SetReadDeadline sets the read deadline associated with the connection.
+func (c *Client) SetReadDeadline(t time.Time) error {
+	return c.c.SetReadDeadline(t)
+}
+
+// SetWriteDeadline sets the write deadline associated with the connection.
+func (c *Client) SetWriteDeadline(t time.Time) error {
+	return c.c.SetWriteDeadline(t)
+}
