@@ -7,6 +7,7 @@ import (
 	"bytes"
 	"crypto/sha1"
 	"encoding/binary"
+	"errors"
 	"net"
 	"os"
 	"time"
@@ -19,6 +20,9 @@ import (
 	"golang.org/x/sys/unix"
 )
 
+// errNotSupported is returned when an operation is not supported
+var ErrNotSupported = errors.New("not supported")
+
 // A client is the Linux implementation of osClient, which makes use of
 // netlink, generic netlink, and nl80211 to provide access to WiFi device
 // actions and statistics.
@@ -116,8 +120,16 @@ func (c *client) Disconnect(ifi *Interface) error {
 // ConnectWPAPSK starts connecting the interface to the specified SSID using
 // WPA.
 func (c *client) ConnectWPAPSK(ifi *Interface, ssid, psk string) error {
+	support, err := c.checkExtFeature(ifi, unix.NL80211_EXT_FEATURE_4WAY_HANDSHAKE_STA_PSK)
+	if err != nil {
+		return err
+	}
+	if !support {
+		return ErrNotSupported
+	}
+
 	// Ask nl80211 to connect to the specified SSID with key..
-	_, err := c.get(
+	_, err = c.get(
 		unix.NL80211_CMD_CONNECT,
 		netlink.Acknowledge,
 		ifi,
@@ -195,6 +207,47 @@ func (c *client) StationInfo(ifi *Interface) ([]*StationInfo, error) {
 	return stations, nil
 }
 
+// SurveyInfo requests that nl80211 return a list of survey information for the
+// specified Interface.
+func (c *client) SurveyInfo(ifi *Interface) ([]*SurveyInfo, error) {
+	msgs, err := c.get(
+		unix.NL80211_CMD_GET_SURVEY,
+		netlink.Dump,
+		ifi,
+		func(ae *netlink.AttributeEncoder) {
+			if ifi.HardwareAddr != nil {
+				ae.Bytes(unix.NL80211_ATTR_MAC, ifi.HardwareAddr)
+			}
+		},
+	)
+	if err != nil {
+		return nil, err
+	}
+
+	surveys := make([]*SurveyInfo, len(msgs))
+	for i := range msgs {
+		if surveys[i], err = parseSurveyInfo(msgs[i].Data); err != nil {
+			return nil, err
+		}
+	}
+	return surveys, nil
+}
+
+// SetDeadline sets the read and write deadlines associated with the connection.
+func (c *client) SetDeadline(t time.Time) error {
+	return c.c.SetDeadline(t)
+}
+
+// SetReadDeadline sets the read deadline associated with the connection.
+func (c *client) SetReadDeadline(t time.Time) error {
+	return c.c.SetReadDeadline(t)
+}
+
+// SetWriteDeadline sets the write deadline associated with the connection.
+func (c *client) SetWriteDeadline(t time.Time) error {
+	return c.c.SetWriteDeadline(t)
+}
+
 // get performs a request/response interaction with nl80211.
 func (c *client) get(
 	cmd uint8,
@@ -524,6 +577,66 @@ func parseRateInfo(b []byte) (*rateInfo, error) {
 	return &info, nil
 }
 
+// parseSurveyInfo parses a single SurveyInfo from a byte slice of netlink
+// attributes.
+func parseSurveyInfo(b []byte) (*SurveyInfo, error) {
+	attrs, err := netlink.UnmarshalAttributes(b)
+	if err != nil {
+		return nil, err
+	}
+
+	var info SurveyInfo
+	for _, a := range attrs {
+		switch a.Type {
+		case unix.NL80211_ATTR_SURVEY_INFO:
+			nattrs, err := netlink.UnmarshalAttributes(a.Data)
+			if err != nil {
+				return nil, err
+			}
+
+			if err := (&info).parseAttributes(nattrs); err != nil {
+				return nil, err
+			}
+
+			// Parsed the necessary data.
+			return &info, nil
+		}
+	}
+
+	// No survey info found
+	return nil, os.ErrNotExist
+}
+
+// parseAttributes parses netlink attributes into a SurveyInfo's fields.
+func (s *SurveyInfo) parseAttributes(attrs []netlink.Attribute) error {
+	for _, a := range attrs {
+		switch a.Type {
+		case unix.NL80211_SURVEY_INFO_FREQUENCY:
+			s.Frequency = int(nlenc.Uint32(a.Data))
+		case unix.NL80211_SURVEY_INFO_NOISE:
+			s.Noise = int(int8(a.Data[0]))
+		case unix.NL80211_SURVEY_INFO_IN_USE:
+			s.InUse = true
+		case unix.NL80211_SURVEY_INFO_TIME:
+			s.ChannelTime = time.Duration(nlenc.Uint64(a.Data)) * time.Millisecond
+		case unix.NL80211_SURVEY_INFO_TIME_BUSY:
+			s.ChannelTimeBusy = time.Duration(nlenc.Uint64(a.Data)) * time.Millisecond
+		case unix.NL80211_SURVEY_INFO_TIME_EXT_BUSY:
+			s.ChannelTimeExtBusy = time.Duration(nlenc.Uint64(a.Data)) * time.Millisecond
+		case unix.NL80211_SURVEY_INFO_TIME_BSS_RX:
+			s.ChannelTimeBssRx = time.Duration(nlenc.Uint64(a.Data)) * time.Millisecond
+		case unix.NL80211_SURVEY_INFO_TIME_RX:
+			s.ChannelTimeRx = time.Duration(nlenc.Uint64(a.Data)) * time.Millisecond
+		case unix.NL80211_SURVEY_INFO_TIME_TX:
+			s.ChannelTimeTx = time.Duration(nlenc.Uint64(a.Data)) * time.Millisecond
+		case unix.NL80211_SURVEY_INFO_TIME_SCAN:
+			s.ChannelTimeScan = time.Duration(nlenc.Uint64(a.Data)) * time.Millisecond
+		}
+	}
+
+	return nil
+}
+
 // attrsContain checks if a slice of netlink attributes contains an attribute
 // with the specified type.
 func attrsContain(attrs []netlink.Attribute, typ uint16) bool {
@@ -574,3 +687,39 @@ func decodeBSSLoad(b []byte) (*BSSLoad, error) {
 	}
 	return &load, nil
 }
+
+// checkExtFeature Checks if a physical interface supports a extended feature
+func (c *client) checkExtFeature(ifi *Interface, feature uint) (bool, error) {
+	msgs, err := c.get(
+		unix.NL80211_CMD_GET_WIPHY,
+		netlink.Dump,
+		ifi,
+		func(ae *netlink.AttributeEncoder) {
+			ae.Flag(unix.NL80211_ATTR_SPLIT_WIPHY_DUMP, true)
+		},
+	)
+	if err != nil {
+		return false, err
+	}
+
+	var features []byte
+found:
+	for i := range msgs {
+		attrs, err := netlink.UnmarshalAttributes(msgs[i].Data)
+		if err != nil {
+			return false, err
+		}
+		for _, a := range attrs {
+			if a.Type == unix.NL80211_ATTR_EXT_FEATURES {
+				features = a.Data
+				break found
+			}
+		}
+	}
+
+	if feature/8 >= uint(len(features)) {
+		return false, nil
+	}
+
+	return (features[feature/8]&(1<<(feature%8)) != 0), nil
+}
