@@ -6,6 +6,7 @@ package wifi
 import (
 	"fmt"
 	"runtime"
+	"time"
 )
 
 // errUnimplemented is returned by all functions on platforms that
@@ -21,6 +22,10 @@ func (*client) Close() error                                     { return errUni
 func (*client) Interfaces() ([]*Interface, error)                { return nil, errUnimplemented }
 func (*client) BSS(_ *Interface) (*BSS, error)                   { return nil, errUnimplemented }
 func (*client) StationInfo(_ *Interface) ([]*StationInfo, error) { return nil, errUnimplemented }
+func (*client) SurveyInfo(_ *Interface) ([]*SurveyInfo, error)   { return nil, errUnimplemented }
 func (*client) Connect(_ *Interface, _ string) error             { return errUnimplemented }
 func (*client) Disconnect(_ *Interface) error                    { return errUnimplemented }
 func (*client) ConnectWPAPSK(_ *Interface, _, _ string) error    { return errUnimplemented }
+func (*client) SetDeadline(t time.Time) error                    { return errUnimplemented }
+func (*client) SetReadDeadline(t time.Time) error                { return errUnimplemented }
+func (*client) SetWriteDeadline(t time.Time) error               { return errUnimplemented }
